package com.adc.da.main.aspect;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UserUtil;
import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.extension.handlers.AbstractSqlParserHandler;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;

@Slf4j
@Intercepts(value = {@Signature(type = Executor.class, method = "update", args = {MappedStatement.class, Object.class})})
public class SqlInterceptor extends AbstractSqlParserHandler implements Interceptor {

    private Logger logger = LoggerFactory.getLogger(getClass());


    /**
     * 创建时间
     */
    private static final String CREATE_TIME = "createTime";
    /**
     * 创建人ID
     */
    private static final String CREATE_USER_ID = "creator";

    /**
     * 更新时间
     */
    private static final String UPDATE_TIME = "updateTime";
    /**
     * 更新人ID
     */
    private static final String UPDATE_USER_ID = "updater";


    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        // SQL操作命令
        SqlCommandType sqlCommandType = mappedStatement.getSqlCommandType();
        // 获取新增或修改的对象参数
        Object parameter = invocation.getArgs()[1];

        if(null == parameter){
            return invocation.proceed();
        }

        // 获取对象中所有的私有成员变量（对应表字段）
        Field[] declaredFields = parameter.getClass().getDeclaredFields();
        if (parameter.getClass().getSuperclass() != null) {
            Field[] superField = parameter.getClass().getSuperclass().getDeclaredFields();
            declaredFields = ArrayUtils.addAll(declaredFields, superField);
        }
        // mybatis plus判断
        boolean plus= parameter.getClass().getDeclaredFields().length == 1 && parameter.getClass().getDeclaredFields()[0].getName().equals("serialVersionUID");

        //兼容mybatis plus
        if (plus) {
            Map<String, Object> updateParam = (Map<String, Object>) parameter;
            if(updateParam.containsKey("param1")&& StringUtils.isNotEmpty(updateParam.get("param1"))){
                Class<?> updateParamType = updateParam.get("param1").getClass();
                declaredFields = updateParamType.getDeclaredFields();
                if (updateParamType.getSuperclass() != null) {
                    Field[] superField = updateParamType.getSuperclass().getDeclaredFields();
                    declaredFields = ArrayUtils.addAll(declaredFields, superField);
                }
            }
        }
        String currentUserId = UserUtil.getCurrentUserId();
//        if(StringUtils.isEmpty(loginUserEO)){
//            throw new AdcDaBaseException("登录失效，请重新登录！");
//        }
        String fieldName = null;
        for (Field field : declaredFields) {
            fieldName = field.getName();
            if (Objects.equals(CREATE_TIME, fieldName)) {
                if (SqlCommandType.INSERT.equals(sqlCommandType)) {
                    try {
                        field.setAccessible(true);
                        field.set(parameter, new Timestamp(System.currentTimeMillis()));
                    } catch (Exception e) {
                        log.warn("添加创建时间报错",e);
                    }
                }
            }
            if (Objects.equals(CREATE_USER_ID, fieldName)&& StringUtils.isNotEmpty(currentUserId)) {
                if (SqlCommandType.INSERT.equals(sqlCommandType)) {
                    field.setAccessible(true);
                    field.set(parameter,currentUserId);
                }
            }

            if (Objects.equals(UPDATE_TIME, fieldName)) {
                if (SqlCommandType.INSERT.equals(sqlCommandType) || SqlCommandType.UPDATE.equals(sqlCommandType)) {
                    field.setAccessible(true);
                    //兼容mybatis plus的update
                    if (plus) {
                        Map<String, Object> updateParam = (Map<String, Object>) parameter;
                        field.set(updateParam.get("param1"), new Timestamp(System.currentTimeMillis()));
                    } else {
                        try {
                            field.set(parameter, new Timestamp(System.currentTimeMillis()));
                        } catch (Exception e) {
                            log.warn("添加修改时间报错",e);
                        }
                    }
                }
            }
            if (Objects.equals(UPDATE_USER_ID, fieldName)&& StringUtils.isNotEmpty(currentUserId) ){
                if (SqlCommandType.INSERT.equals(sqlCommandType) || SqlCommandType.UPDATE.equals(sqlCommandType)) {
                    field.setAccessible(true);
                    //兼容mybatis plus的update
                    if (plus) {
                        Map<String, Object> updateParam = (Map<String, Object>) parameter;
                        field.set(updateParam.get("param1"), currentUserId);
                    } else {
                        field.set(parameter, currentUserId);
                    }
                }
            }
        }
//        //校验登录用户
//        for(Method m : ms){
//            if(m.getName().equals(methedName)){
////获取注解  来判断是不是要储存sql
//                SqlLogs annotation = m.getAnnotation(SqlLogs.class);
//                if(annotation != null){
//                    hasSqlLogs = annotation.hasSqlLog();
//                }
//            }
//        }
        return invocation.proceed();
    }

    /**
     * 判断之前sql是否存在了代理商查询字段
     *
     * @param
     * @return boolean
     * @author: ZhiHao
     * @date: 2020/12/21
     */
//    private boolean doesItExist(Expression sql) {
//        String str = sql != null ? sql.toString() : null;
//        if (StringUtils.containsIgnoreCase(str, TenantTable.AGENT_ID.getColumns(null))
//                || StringUtils.containsIgnoreCase(str, TenantTable.DEPOT_ID.getColumns(null))) {
//            return false;
//        }
//        return true;
//    }


//   void proceedUserAuth(Invocation invocation) throws ClassNotFoundException {
//        Map<String,Object> sqlMap = new HashMap<>();
//        Boolean userAuth = false;
//        StatementHandler statementHandler = PluginUtils.realTarget(invocation.getTarget());
//        MetaObject metaObject = SystemMetaObject.forObject(statementHandler);
//        this.sqlParser(metaObject);
//        MappedStatement mappedStatement =
//                (MappedStatement) metaObject.getValue("delegate.mappedStatement");
//
//        final Object[] args = invocation.getArgs();
//        //获取执行方法的位置
//        String namespace = mappedStatement.getId();
//        //获取mapper名称
//        String className = namespace.substring(0,namespace.lastIndexOf("."));
//        //获取方法名
//        String methedName= namespace.substring(namespace.lastIndexOf(".") + 1,namespace.length());
//        //获取当前mapper 的方法
//        Method[] ms = Class.forName(className).getMethods();
//        for(Method m : ms){
//            if(m.getName().equals(methedName)){
//         //获取注解  来判断是不是要储存sql
//                UserAuth annotation = m.getAnnotation(UserAuth.class);
//                if(annotation != null){
//                    userAuth = annotation.loginFilter();
//                }
//            }
//        }
//        //如果是有注解值为true，便获取sql处理参数
//        if(userAuth){
//            // 执行的SQL语句
////            BoundSql boundSql = (BoundSql) metaStatementHandler.getValue("delegate.boundSql");
////            String sql = boundSql.getSql();
////            logger.info("之前sql语句:{}", sql);
////            // 判断是否符合需要增加区分代理商查询条件
////            sql = this.ifAgentQuery(sql);
////            logger.info("代理商查询sql语句:{}", sql);
////            // 最终将修改好的sql语句设置回去执行
////            metaStatementHandler.setValue("delegate.boundSql.sql", sql);
//
//        }
//    }

//    /**
//     * 判断是否需要进行拼接代理商查询条件
//     * 有以下几种情况: (0 运营商, 不进行拼接查询全部) , (1 代理商并且是总仓管理员, 仅拼接代理商字段查询)
//     * (2 代理商并且是分仓管理员, 需拼接代理商字段与仓库字段查询) , (3 代理商并且是总仓管理员, 需拼接代理商字段查询)
//     *
//     * @param sql
//     * @return java.lang.String
//     * @author: ZhiHao
//     * @date: 2020/12/17
//     */
//    private String ifAgentQuery(String sql) {
//        // sql解析器解析查询语句(也只有查询能进来)
//        Select selectStatement = (Select) CCJSqlParserUtil.parse(sql);
//        // 不是单表与多表直接查询
//        if (!(selectStatement.getSelectBody() instanceof PlainSelect)) {
//            return sql;
//        }
//        PlainSelect selectBody = (PlainSelect) selectStatement.getSelectBody();
//        // 仅对枚举符合表, 并没有代理商字段的进行拼接sql
//        Table table = this.ifConform(selectBody);
//        if (table != null && this.doesItExist(selectBody.getWhere())) {
//            // 进行拼接, 判断是否有where
//            Expression where = selectBody.getWhere();
//            // 获取查询语句表别名(兼容之前做好功能一个方法表别名不一致)
//            String sqlTableAlias = table.getAlias() != null ? table.getAlias().getName() : null;
//            String queryConditionsAndValues;
//            if (where != null) {
//                // 获取代理商查询条件与值
//                queryConditionsAndValues = this.getQueryConditionsAndValues( table.getName(),
//                        StringUtils.isNotBlank(sqlTableAlias) ? sqlTableAlias : this.tableAlias);
//                // 解析之前表达式, 使用弱引用
//                WeakReference<ExpressionDeParser> weakReference = new WeakReference<>(new ExpressionDeParser());
//                where.accept(weakReference.get());
//                // 获取之前表达式
//                StringBuilder buffer = weakReference.get().getBuffer();
//                // 拼接sql
//                buffer.append(queryConditionsAndValues);
//                // 设置回去
//                selectBody.setWhere(CCJSqlParserUtil.parseCondExpression(buffer.toString()));
//                weakReference.clear();
//            } else {
//                // 获取查询条件与值
//                queryConditionsAndValues = " 1 = 1 " + this.getQueryConditionsAndValues(table.getName(),
//                        StringUtils.isNotBlank(sqlTableAlias) ? sqlTableAlias : this.tableAlias);
//                //没有where情况
//                selectBody.setWhere(CCJSqlParserUtil.parseCondExpression(queryConditionsAndValues));
//            }
//            sql = selectStatement.toString();
//        }
//        return sql;
//    }


    @Override
    public Object plugin(Object target) {
        if (target instanceof Executor) {
            return Plugin.wrap(target, this);
        }
        return target;
    }

    @Override
    public void setProperties(Properties properties) {
    }
}
