package com.adc.da.main.config;


import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.util.StringUtil;
import com.baomidou.mybatisplus.extension.plugins.tenant.TenantHandler;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.NullValue;
import net.sf.jsqlparser.expression.StringValue;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

@Slf4j
@Component
public class CustomTenantHandler implements TenantHandler {
    //需要过滤多租户的表
    List<String> filterTable = Arrays.asList("CERT_CONFIG_SEQ",
            "CERT_CONFIG_SEQ_TEMPLATE",
            "CERT_HGZ_PRINT",
            "VEHICLE_CERT",
            "CERT_ZK",
            "CERT_PRINTER");

    /**
     * 租户Id
     *
     * @return
     */
    @Override
    public Expression getTenantId(boolean where) {
        //可以通过过滤器从请求中获取对应租户id
        String tenantId = UserUtil.getCurrentUserName();
        log.debug("当前租户为{}", tenantId);
        if (StringUtil.isEmpty(tenantId)) {
            return new NullValue();
        }
        return new StringValue(tenantId);
    }

    /**
     * 租户字段名
     *
     * @return
     */
    @Override
    public String getTenantIdColumn() {
        return "QY_ID";
    }

    /**
     * 根据表名判断是否进行过滤
     * 忽略掉一些表：如租户表（sys_tenant）本身不需要执行这样的处理
     *
     * @param tableName
     * @return
     */
    @Override
    public boolean doTableFilter(String tableName) {
        if (filterTable.contains(tableName)) {
            return false;
        }
        return true;
    }


}
